@@ -6,13 +6,15 @@ import os
 from pathlib import Path
 import logging
 from urllib.parse import urlparse
-from typing import List, Iterator, Dict, Any
+from typing import List, Iterator, Dict, Any, Optional
 from scrapy import Spider  # type: ignore
 from scrapy.linkextractors import LinkExtractor  # type: ignore
 from scrapy.crawler import CrawlerProcess, Crawler  # type: ignore
 from scrapy import signals
 from scrapy_splash import SplashRequest, SplashJsonResponse  # type: ignore
 
+from .exceptions import InvalidProxy
+
 # Mapping from https://doc.qt.io/qt-5/qnetworkreply.html#NetworkError-enum
 error_mapping = {
     'http': "HTTP Error {}",
@@ -58,15 +60,32 @@ class ScrapySplashWrapperCrawler():
         name = 'ScrapySplashWrapperSpider'
         handle_httpstatus_all = True  # https://docs.scrapy.org/en/latest/topics/spider-middleware.html?highlight=handle_httpstatus_all#std-reqmeta-handle_httpstatus_all
 
-        def __init__(self, url: str, useragent: str, cookies: List[Dict[Any, Any]]=[], referer: str='', log_level: str='WARNING', *args, **kwargs) -> None:
+        def __init__(self, url: str, useragent: str, cookies: Optional[List[Dict[Any, Any]]]=None, referer: str='', proxy: str='', log_level: str='WARNING', *args: Any, **kwargs: Any) -> None:
             logger = logging.getLogger('scrapy')
             logger.setLevel(log_level)
             super().__init__(*args, **kwargs)
             self.start_url: str = url
             self.useragent: str = useragent
             self.allowed_domains: List[str] = []
+            if cookies is None:
+                cookies = []
             self.cookies: List[Dict[Any, Any]] = cookies
             self.referer: str = referer
+            self.proxy: Dict[str, str] = {}
+            if proxy:
+                parsed_proxy = urlparse(proxy)
+                self.proxy['host'] = parsed_proxy.hostname if parsed_proxy.hostname else ''
+                self.proxy['port'] = str(parsed_proxy.port) if parsed_proxy.port else ''
+                self.proxy['type'] = parsed_proxy.scheme if parsed_proxy.scheme else ''
+                self.proxy['username'] = parsed_proxy.username if parsed_proxy.username else ''
+                self.proxy['password'] = parsed_proxy.password if parsed_proxy.password else ''
+                if not all((self.proxy['host'], self.proxy['port'], self.proxy['type'])):
+                    # If we have a proxy, host, port and type are required
+                    raise InvalidProxy(proxy, 'A valid proxy requires at least a type, hostname and port.')
+
+                if any((self.proxy['username'], self.proxy['password'])) and not all((self.proxy['username'], self.proxy['password'])):
+                    raise InvalidProxy(proxy, 'If the proxy requires a username, it also requires a password.')
+
             hostname = urlparse(self.start_url).hostname
             if hostname:
                 self.allowed_domains = ['.'.join(hostname.split('.')[-2:])]
@@ -74,13 +93,14 @@ class ScrapySplashWrapperCrawler():
             with (realpath / 'crawl.lua').open() as _crawl:
                 self.script = _crawl.read()
 
-        def start_requests(self):
+        def start_requests(self) -> Iterator[SplashRequest]:
             yield SplashRequest(self.start_url, self.parse, endpoint='execute',
                                 args={'wait': 15, 'resource_timeout': 40,
                                       'timeout': 90,
                                       'useragent': self.useragent,
                                       'referer': self.referer,
                                       'cookies': [{k: v for k, v in cookie.items() if v is not None} for cookie in self.cookies],
+                                      'myproxy': self.proxy,
                                       'lua_source': self.script
                                       })
 
@@ -92,14 +112,18 @@ class ScrapySplashWrapperCrawler():
                                           'useragent': self.useragent,
                                           'referer': response.data['last_redirected_url'],
                                           'cookies': response.data['cookies'],
+                                          'myproxy': self.proxy,
                                           'lua_source': self.script
                                           })
             yield response.data
 
-    def __init__(self, *, splash_url: str, useragent: str, cookies: List[Dict[Any, Any]]=[], referer: str='', depth: int=1, log_enabled: bool=False, log_level: str='WARNING'):
+    def __init__(self, *, splash_url: str, useragent: str, cookies: Optional[List[Dict[Any, Any]]]=None, referer: str='', proxy: str='', depth: int=1, log_enabled: bool=False, log_level: str='WARNING'):
         self.useragent = useragent
+        if cookies is None:
+            cookies = []
         self.cookies = cookies
         self.referer = referer
+        self.proxy = proxy.strip()
         self.log_level = log_level
         self.process = CrawlerProcess({'LOG_ENABLED': log_enabled})
         self.crawler = Crawler(self.ScrapySplashWrapperSpider, {
@@ -119,7 +143,7 @@ class ScrapySplashWrapperCrawler():
     def crawl(self, url: str) -> List[Any]:
         crawled_items = []
 
-        def add_item(item) -> None:
+        def add_item(item: Dict[str, Any]) -> None:
             if 'error' in item:
                 key = item.pop('error')
                 error = {'key': key, 'details': "Unknown error, see https://doc.qt.io/qt-5/qnetworkreply.html#NetworkError-enum"}
@@ -134,6 +158,6 @@ class ScrapySplashWrapperCrawler():
             crawled_items.append(item)
 
         self.crawler.signals.connect(add_item, signals.item_scraped)
-        self.process.crawl(self.crawler, url=url, useragent=self.useragent, cookies=self.cookies, referer=self.referer, log_level=self.log_level)
+        self.process.crawl(self.crawler, url=url, useragent=self.useragent, cookies=self.cookies, referer=self.referer, proxy=self.proxy, log_level=self.log_level)
         self.process.start()
         return crawled_items
